from __future__ import absolute_import
from aspen import renderers
from tornado.template import Loader, Template
import os


class Renderer(renderers.Renderer):

    def compile(self, filepath, raw):
        loader = self.meta
        return Template(raw, filepath, loader, compress_whitespace=False)

    def render_content(self, context):
        return self.compiled.generate(**context)


class Factory(renderers.Factory):

    Renderer = Renderer

    def compile_meta(self, configuration):
        bases_dir = configuration.project_root
        if bases_dir is None:
            loader = None
        else:
            loader = LoaderShim(bases_dir)
        return loader


class LoaderShim(Loader):
    """
    The Tornado loader doesn't properly detect absolute Windows paths when
    resolving a template's path. This shim adds portable absolute path
    detection, falling back to the original Loader for relative paths.
    """

    def resolve_path(self, name, parent_path=None):
        # This is the inverse of the test Tornado's Loader.resolve_path() does
        # on a template's name and its parent's name. Tornado only tests for
        # absolute paths by looking for '/', which fails on Windows. If it
        # detects an absolute path it returns the name unmodified, so we defer
        # to the Tornado implementation if the path isn't absolute.
        if not parent_path or parent_path.startswith("<") or \
                os.path.isabs(parent_path) or os.path.isabs(name):
            return name

        return super(LoaderShim, self).resolve_path(name, parent_path)



"""Implement a Jinja2 renderer.

Jinja2 insists on unicode, and explicit loader objects. We assume with Jinja2
that your templates on the filesystem be encoded in UTF-8 (the result of the
template will be encoded to bytes for the wire per response.charset). We shim a
loader that returns the decoded content page and instructs Jinja2 not to
perform auto-reloading.

"""
from __future__ import absolute_import
from aspen import renderers

from jinja2 import BaseLoader, Environment, FileSystemLoader


class SimplateLoader(BaseLoader):
    """Jinja2 really wants to get templates via a Loader object.

    See: http://jinja.pocoo.org/docs/api/#loaders

    """

    def __init__(self, filepath, raw):
        self.filepath = filepath
        self.decoded = raw.decode('UTF-8')

    def get_source(self, environment, template):
        return self.decoded, self.filepath, True


class Renderer(renderers.Renderer):

    def compile(self, filepath, raw):
        environment = self.meta
        return SimplateLoader(filepath, raw).load(environment, filepath)

    def render_content(self, context):
        charset = context['response'].charset
        return self.compiled.render(context).encode(charset)


class Factory(renderers.Factory):

    Renderer = Renderer

    def compile_meta(self, configuration):
        loader = None
        if configuration.project_root is not None:
            # Instantiate a loader that will be used to resolve template bases.
            loader = FileSystemLoader(configuration.project_root)
        return Environment(loader=loader)



from distribute_setup import use_setuptools
use_setuptools()

from setuptools import find_packages, setup
version = open('version.txt').read()


classifiers = [ 'Development Status :: 4 - Beta'
              , 'Environment :: Console'
              , 'Intended Audience :: Developers'
              , 'License :: OSI Approved :: MIT License'
              , 'Natural Language :: English'
              , 'Operating System :: MacOS :: MacOS X'
              , 'Operating System :: Microsoft :: Windows'
              , 'Operating System :: POSIX'
              , 'Programming Language :: Python :: 2.5'
              , 'Programming Language :: Python :: 2.6'
              , 'Programming Language :: Python :: 2.7'
              , 'Programming Language :: Python :: Implementation :: CPython'
              , 'Programming Language :: Python :: Implementation :: Jython'
              , 'Topic :: Internet :: WWW/HTTP :: HTTP Servers'
               ]

setup( author = 'Chad Whitacre'
     , author_email = 'chad@zetaweb.com'
     , classifiers = classifiers
     , description = ('Aspen is a Python web framework. '
                      'Simplates are the main attraction.')
     , entry_points = { 'console_scripts': [ 'aspen = aspen.server:main'
                                           , 'thrash = thrash:main'
                                           , 'swaddle = swaddle:main'
                                           , 'fcgi_aspen = fcgi_aspen:main'
                                            ] }
     , name = 'aspen'
     , packages = find_packages(exclude=[ 'aspen.tests'
                                        , 'aspen.tests.*'
                                         ])
     , py_modules = ['swaddle', 'thrash', 'fcgi_aspen']
     , url = 'http://aspen.io/'
     , version = version
     , zip_safe = False
     , package_data = {'aspen': [ 'www/*'
                                , 'configuration/mime.types'
                                 ]}
     , install_requires = [ 'Cheroot==4.0.0beta'
                          , 'mimeparse==0.1.3'
                          , 'tornado==1.2.1'
                           ]
      )
